//地球磁場中での荷電粒子の運動
#include <stdio.h>
#include <math.h>

#define PARTICLE_NUMBER 1
const double e = 1.602176565e-19; // Elementary charge (Coulomb)
const double m_pr = 1.672621777e-27; // Proton mass (kg)
const double m_el = 9.10938291e-31; // Electron mass (kg)
const double c = 299792458; // speed of light (m/s)
const double Re = 6378137; // Earth radius (m)

struct particle {
    double m[PARTICLE_NUMBER]; // mass
    double x[PARTICLE_NUMBER];
    double y[PARTICLE_NUMBER];
    double z[PARTICLE_NUMBER];
    double vx[PARTICLE_NUMBER];
    double vy[PARTICLE_NUMBER];
    double vz[PARTICLE_NUMBER];
};

// magnetic field
struct mf {
    double Bx, By, Bz;
};

//磁場双極子
void dipole(double x, double y, double z, struct mf *b);

int main(void) {
    for (int j=0; j<PARTICLE_NUMBER; j++) {
        FILE *fp;
        fp=fopen("aurora4.txt", "w");
        fprintf(fp, "#x\t\ty\t\tz\n");

        double t=0.0, dt=0.0001; //2*Reの距離でも正確に計算できるように刻み幅を設定
        struct particle p;
        struct mf b;
        p.m[j]=m_pr;
        p.x[j]=-4*Re;
        //p.x[j]=-40*Re; //太陽風
        p.y[j]=0.0;
        p.z[j]=0.0;
        p.vx[j]=0.1*c;
        //p.vx[j]=0.001*c; //太陽風
        p.vy[j]=0.0;
        p.vz[j]=0.0;

        for (int i=0; i<=1000000; i++) { //太陽風 i<=100000000 dt=0.001
            t=i*dt;
            if (i%100==0) { //データを間引く
                fprintf(fp, "%f\t%f\t%f\n", p.x[j]/Re, p.y[j]/Re, p.z[j]/Re);
            }
            // Runge-Kutta coefficients
            double k1x, k1y, k1z, k1vx, k1vy, k1vz;
            double k2x, k2y, k2z, k2vx, k2vy, k2vz;
            double k3x, k3y, k3z, k3vx, k3vy, k3vz;
            double k4x, k4y, k4z, k4vx, k4vy, k4vz;
            double vxtmp, vytmp, vztmp;

            vxtmp = p.vx[j];
            vytmp = p.vy[j];
            vztmp = p.vz[j];
            k1x=dt*(vxtmp);
            k1y=dt*(vytmp);
            k1z=dt*(vztmp);
            dipole(p.x[j], p.y[j], p.z[j], &b);
            k1vx=dt*e/p.m[j]*(vytmp*b.Bz-vztmp*b.By);
            k1vy=dt*e/p.m[j]*(vztmp*b.Bx-vxtmp*b.Bz);
            k1vz=dt*e/p.m[j]*(vxtmp*b.By-vytmp*b.Bx);

            vxtmp = p.vx[j]+k1vx/2.0;
            vytmp = p.vy[j]+k1vy/2.0;
            vztmp = p.vz[j]+k1vz/2.0;
            k2x=dt*(vxtmp);
            k2y=dt*(vytmp);
            k2z=dt*(vztmp);
            dipole(p.x[j], p.y[j], p.z[j], &b);
            k2vx=dt*e/p.m[j]*(vytmp*b.Bz-vztmp*b.By);
            k2vy=dt*e/p.m[j]*(vztmp*b.Bx-vxtmp*b.Bz);
            k2vz=dt*e/p.m[j]*(vxtmp*b.By-vytmp*b.Bx);

            vxtmp = p.vx[j]+k2vx/2.0;
            vytmp = p.vy[j]+k2vy/2.0;
            vztmp = p.vz[j]+k2vz/2.0;
            k3x=dt*(vxtmp);
            k3y=dt*(vytmp);
            k3z=dt*(vztmp);
            dipole(p.x[j], p.y[j], p.z[j], &b);
            k3vx=dt*e/p.m[j]*(vytmp*b.Bz-vztmp*b.By);
            k3vy=dt*e/p.m[j]*(vztmp*b.Bx-vxtmp*b.Bz);
            k3vz=dt*e/p.m[j]*(vxtmp*b.By-vytmp*b.Bx);

            vxtmp = p.vx[j]+k3vx;
            vytmp = p.vy[j]+k3vy;
            vztmp = p.vz[j]+k3vz;
            k4x=dt*(vxtmp);
            k4y=dt*(vytmp);
            k4z=dt*(vztmp);
            dipole(p.x[j], p.y[j], p.z[j], &b);
            k4vx=dt*e/p.m[j]*(vytmp*b.Bz-vztmp*b.By);
            k4vy=dt*e/p.m[j]*(vztmp*b.Bx-vxtmp*b.Bz);
            k4vz=dt*e/p.m[j]*(vxtmp*b.By-vytmp*b.Bx);

            p.x[j]=p.x[j]+(k1x + 2.0*k2x + 2.0*k3x + k4x)/6.0;
            p.y[j]=p.y[j]+(k1y + 2.0*k2y + 2.0*k3y + k4y)/6.0;
            p.z[j]=p.z[j]+(k1z + 2.0*k2z + 2.0*k3z + k4z)/6.0;
            p.vx[j]=p.vx[j]+(k1vx + 2.0*k2vx + 2.0*k3vx + k4vx)/6.0;
            p.vy[j]=p.vy[j]+(k1vy + 2.0*k2vy + 2.0*k3vy + k4vy)/6.0;
            p.vz[j]=p.vz[j]+(k1vz + 2.0*k2vz + 2.0*k3vz + k4vz)/6.0;
        }
        fclose(fp);
    }
    return 0;
}

void dipole(double x, double y, double z, struct mf *b) {
    double Btheta, Br;
    double mu0=4*M_PI*1.0e-7, m=7.768e+22, r, sintheta, costheta, R, sinphi, cosphi, BR;
    double omega=23.4; // 23.4
    double rotate[2][2];
    rotate[0][0]=cos(omega*M_PI/180);
    rotate[0][1]=-sin(omega*M_PI/180);
    rotate[1][0]=sin(omega*M_PI/180);
    rotate[1][1]=cos(omega*M_PI/180);
    double x_new, z_new, Bx_new, Bz_new;

    x_new=x*rotate[0][0]+z*rotate[0][1];
    z_new=x*rotate[1][0]+z*rotate[1][1];
    x=x_new;
    z=z_new;
    R=x*x+y*y;
    R=sqrt(R);
    if (R!=0) {
        sinphi=y/R;
        cosphi=x/R;
    } else {
        sinphi=0.0;
        cosphi=0.0;
    }
    r=R*R+z*z;
    r=sqrt(r);
    if (r!=0) {
        sintheta=R/r; //回転半径Rを使う
        costheta=z/r;
        Btheta=mu0*m*sintheta/(4*M_PI*r*r*r);
        Br=mu0*2*m*costheta/(4*M_PI*r*r*r);
    } else {
        sintheta=0.0;
        costheta=0.0;
        Btheta=0.0;
        Br=0.0;
    }
    BR=Btheta*costheta+Br*sintheta;
    b->Bz=Btheta*(-sintheta)+Br*costheta;

    b->Bx=BR*cosphi;
    b->By=BR*sinphi;

    //逆回転行列
    Bx_new=b->Bx*rotate[0][0]+b->Bz*rotate[1][0];
    Bz_new=b->Bx*rotate[0][1]+b->Bz*rotate[1][1];
    b->Bx=Bx_new;
    b->Bz=Bz_new;
    b->Bx=b->Bx*(-1);
    b->By=b->By*(-1);
    b->Bz=b->Bz*(-1);
}
